#ifndef QT_FEATURES_NetworkAuth_src_oauth_qtnetworkauth_config_p_h_H
#define QT_FEATURES_NetworkAuth_src_oauth_qtnetworkauth_config_p_h_H

#define QT_FEATURE_urischeme_replyhandler 1


#endif // QT_FEATURES_NetworkAuth_src_oauth_qtnetworkauth_config_p_h_H
